'' =================================================================================================
''
''   File....... jm_pasmblink_demo.spin
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2017 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 11 MAR 2017
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }

  RX1  = 31                                                     ' programming / terminal
  TX1  = 30

  SDA1 = 29                                                     ' eeprom / i2c
  SCL1 = 28

  LED1 = 27
  LED0 = 26


obj

' main                                                          ' * master Spin cog
  time : "jm_time"                                              '   timing and delays
  io   : "jm_io_basic"                                          '   essential io
  term : "jm_fullduplexserial"                                  ' * serial IO for terminal
                                                                 
' * uses cog when loaded                                         
                                                                 
                                                                 
var                                                              
                                                                 
                                                                 
pub main                                                         
                                                                 
  setup                                                          
                                                                 
  blinker_start(LED0, 250, LED1, 1000)    
                                                                 
  repeat                                                         
    waitcnt(0)                                                   
                                                                 
                                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)
                                                                 
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal
                                                                 

con

  { ------------------------------------- }
  {  O B J E C T   D E V E L O P M E N T  }
  {  -- move to file when tested          }
  { ------------------------------------- }


var

  long  cog

  long  pins
  long  param0
  long  param1


pub blinker_start(led0pin, led0ms, led1pin, led1ms)

'' Starts dual blinker cog

  blinker_stop

  pins   := (led1pin << 8) | led0pin                            ' compress pins
  param0 := led0ms * MS_001                                     ' convert ms to ticks
  param1 := led1ms * MS_001

  cog := cognew(@entry, @pins) + 1                              ' start the cog

  return cog  


pub blinker_stop

  if (cog)
    cogstop(cog + 1)
    cog := 0


dat { dual blinker cog }

                        org     0
entry
b0addr  if_never        mov     b0addr, #blink0                 ' assign task addresses 
b1addr  if_never        mov     b1addr, #blink1

pasm_setup              mov     t1, par                         ' start of parameters
                        rdlong  t2, t1                          ' read pins
                        mov     t3, t2                          ' make copy

cfg_led0                and     t2, #$1F                        ' extract led 0 pin
                        mov     led0mask, #1                    ' convert to mask
                        shl     led0mask, t2
                        or      dira, led0mask                  ' set to output

cfg_led1                shr     t3, #8                          ' extract led 1 pin
                        and     t3, #$1F       
                        mov     led1mask, #1   
                        shl     led1mask, t3   
                        or      dira, led1mask

get_timing              add     t1, #4                          
                        rdlong  led0tix, t1                     ' read led 0 timing
                        add     t1, #4
                        rdlong  led1tix, t1                     ' read led 1 timing

init_targets            mov     led0tgt, led0tix                ' initialize toggle targets
                        add     led0tgt, cnt
                        mov     led1tgt, led1tix
                        add     led1tgt, cnt

                        
blink0                  jmpret  b0addr, b1addr                  ' run blink2 then come back
blink0x                 mov     t1, led0tgt                     ' get target time
                        sub     t1, cnt                         ' subtract cnt from it
                        cmps    t1, #0                  wc      ' are we there yet?
        if_nc           jmp     #blink0                         ' if no, back to top
                        xor     outa, led0mask                  ' invert output
                        add     led0tgt, led0tix                ' update timer
                        jmp     #blink0                         ' back to top

                        
blink1                  jmpret  b1addr, b0addr
blink1x                 mov     t1, led1tgt
                        sub     t1, cnt
                        cmps    t1, #0                  wc
        if_nc           jmp     #blink1
                        xor     outa, led1mask
                        add     led1tgt, led1tix
                        jmp     #blink1

' --------------------------------------------------------------------------------------------------

led0mask                res     1                               ' mask for LED0 pin
led0tix                 res     1                               ' ticks in LED0 half cycle
led0tgt                 res     1                               ' target (in cnt) for LED toggle                              

led1mask                res     1
led1tix                 res     1
led1tgt                 res     1

t1                      res     1                               ' work vars
t2                      res     1
t3                      res     1

                        fit     496

                 
dat { license }

  Copyright     byte    "Copyright (C) 2017 Jon McPhalen", 0                                                 

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      