'' =================================================================================================
''
''   File....... jm_sprite_demo.spin 
''   Purpose....
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (C) 2017 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 19 MAR 2017
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x                                     
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us


con { io pins }                                                  
                                                                 
  RX1     = 31  { I }                                           ' serial / programming
  TX1     = 30  { O }                                            
                                                                 
  SDA     = 29  { I/O }                                         ' i2c / eeprom
  SCL     = 28  { I/O }                                          
                                                                 
  XP0     = 27  { I/O }                                         ' extra port 0 - not on Rev E or lower
  XP1     = 24  { I/O }                                         ' extra port 1 - not on Rev E or lower 
  XP2     = 19  { I/O }                                         ' extra port 2 - not on Rev E or lower    
  XP3     = 16  { I/O }                                         ' extra port 3 - not on Rev E or lower 
                                                                 
  SIO_RX  = 26  { I/O }                                         ' TTL serial io
  SIO_TX  = 25  { I/O }                                         ' TTL serial io (not enabled)  
                                                                 
  RX2     = 23  { I }                                           ' RS-485 port
  RX2_EN  = 22  { O }                                                
  TX2_EN  = 21  { O }                                                
  TX2     = 20  { O }                                                
  
  G_LED   = 18  { O }                                           ' bi-color LED (Rev F assignments)
  R_LED   = 17  { O }                                              
                                                                 
  OUT7    = 15  { O }                                           ' outputs
  OUT6    = 14  { O }                                                
  OUT5    = 13  { O }                                                
  OUT4    = 12  { O }                                                
  OUT3    = 11  { O }                                                
  OUT2    = 10  { O }                                                
  OUT1    =  9  { O }                                                
  OUT0    =  8  { O }                                                
                                                                 
  OPT_BR  =  7  { I/O }                                         ' options switches
  OPT_A1  =  6  { I/O }                                                 
  OPT_A0  =  5  { I/O }                                                 
  OPT_SM  =  4  { I/O }                                                 
                                                                 
  DMX_A8  =  3  { I }                                           ' high-bit of DMX address
                                                                 
  DO_165  =  2  { I }                                           ' control lines for 74x165
  CLK_165 =  1  { O }                                                 
  LD_165  =  0  { O } 

  
con { app io }

  RX3     =  OPT_A0                                             ' from Sprite TX
  TX3     =  OPT_SM                                             ' to Sprite RX


obj

' main                                                          ' * master Spin cog
  time   : "jm_time_80"                                         '   timing and delays
  io     : "jm_io_basic"                                        '   essential io
  sprite : "jm_sprite"                                          ' * serial IO for sprite  
  term   : "jm_fullduplexserial"                                ' * serial IO for terminal
                                                                 
                                                                 
var
                                                          
                                                                 
pub main | last1, last2, t, c1, c2                                                 
                                                                 
  setup

  term.rxflush

  last1 := last2 := -1

  t := time.millis
  repeat
    c1 := sprite.current_file
    c2 := sprite.alert
    if ((c1 <> last1) or (c2 <> last2))                         ' status change?
      t := time.millis                                          ' reset timer   
      last1 := c1                                               ' save for next check
      last2 := c2                                             
      report(c1, c2, sprite.millis)                             ' show status
      
    if ((time.millis - t) => 100)                               ' 100ms elapsed since last report?
      report(-1, -1, sprite.millis)                             ' report current state                               
      t += 100
    
    c1 := term.rxcheck                                          ' input from terminal?
    if ((c1 => "1") and (c1 =< "9"))                            ' if file #
      sprite.command(c1-"0")                                    '  play it
    elseif (c1 == " ")                                          ' if space bar
      sprite.command(sprite#KEY_PAUSE)                          '  pause/resume


pub report(fn, xb, now) 

  if (fn < 0)
    fn := sprite.current_file

  term.hex(fn, 2)                                               ' file number
  term.tx(term#TAB) 

  if (xb < 0)
    xb := sprite.alert

  term.hex(xb, 2)                                               ' specialty byte
  term.tx(term#TAB)

  show_time(now)
  term.tx(term#CR)   


pub show_time(ms) | ss, mn

  ss := ms / 1000
  ms //= 1000

  mn := ss / 60
  ss := ss // 60

  term.rjdec(mn, 2, "0")
  term.tx(":")
  term.rjdec(ss, 2, "0") 
  term.tx(".")
  term.rjdec(ms, 3, "0") 


pub dec2(value)

  term.tx("0" + value  / 10)
  term.tx("0" + value // 10)

                                                 
pub setup                                                        
                                                                 
'' Setup IO and objects for application                          
                                                                 
  time.start                                                    ' setup timing & delays

  io.start(0, 0)                                                ' clear all pins (master cog)

  sprite.start(RX3, TX3, 9600)                                  ' start serial for sprite *
                                                             
  term.start(RX1, TX1, %0000, 115_200)                          ' start serial for terminal *
                                                                 
                  
dat { license }

  Copyright     byte    "Copyright (C) 2017 Jon McPhalen", 0                                                 

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 

}}      